// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/
#include <stdlib.h>
#include <limits.h>

template<class T> xdata::IntegerType<T>::IntegerType(T value)
:xdata::SimpleType<T>(value)
{
}

template<class T> xdata::IntegerType<T>::IntegerType()
{
}

template<class T> xdata::IntegerType<T>::IntegerType(const std::string & value)

: xdata::SimpleType<T>(value)
{
	this->fromString(value);
}

template<class T> xdata::IntegerType<T>::~IntegerType()
{
}

template<class T> std::string xdata::IntegerType<T>::type() const
{
		return "int";
}

template<class T> std::string xdata::IntegerType<T>::toString () const 
{
	if (this->isInf())
	{
		return "infinite";
	}
	else if (this->isNaN())
	{
		return "NaN";
	}
	else
	{
		char tmp[255];
		sprintf (tmp, "%d", this->value_);
		return tmp;
	}
}

template<class T> void  xdata::IntegerType<T>::fromString (const std::string & value) 

{	
	this->limits_.reset(); // reset any bits
	if ((value == "") || (value == "NaN"))
	{
		// empty string is not a number
		this->limits_.set(xdata::NaN);
		return;
	}
	else if (value == "infinite")
	{
		this->limits_.set(xdata::Infinity);
		return;
	}
	// The following code will convert an integer
	// from any represenation, e.g. hex, oct, bin or decimal
	//
	char* firstWrong = 0;

	errno = 0;
	long int tmp = strtol ( value.c_str(), &firstWrong, 0 );
	if (	(tmp <= INT_MIN) || 
		(tmp >= INT_MAX) ||
		(*firstWrong != '\0') || (errno == ERANGE))
	{
		this->limits_.set(xdata::NaN);
		std::string msg = "Invalid long integer format. Value ";
		msg = value;
		msg += ", error at ";
		msg += firstWrong;
		XCEPT_RAISE (xdata::exception::Exception, msg);
	}
	this->value_ = static_cast<int>(tmp);
}	

template<class T> T& xdata::IntegerType<T>::operator++()
{
	if ( std::numeric_limits<xdata::Integer>::max()  == this->value_ )
	{
		this->limits_.set(xdata::Infinity);
		return this->value_;
	}
	else
	{
		return (++this->value_);
	}
}

template<class T> T& xdata::IntegerType<T>::operator--()
{
	if ( std::numeric_limits<xdata::Integer>::min()  == this->value_ )
	{
		this->limits_.set(xdata::Infinity);
		return this->value_;
	}
	else
	{
		return (--this->value_);
	}	
}

template<class T> T xdata::IntegerType<T>::operator++(int)
{
	if ( std::numeric_limits<xdata::Integer>::max()  == this->value_ )
	{
		this->limits_.set(xdata::Infinity);
		return this->value_;
	}
	else
	{
		this->value_++;
		return this->value_;
	}
}

template<class T> T xdata::IntegerType<T>::operator--(int)
{
	if ( std::numeric_limits<xdata::Integer>::min()  == this->value_ )
	{
		this->limits_.set(xdata::Infinity);
		return this->value_;
	}
	else
	{
		this->value_--;
		return this->value_;
	}	
}

