// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/

#ifndef _xdata_String_h_
#define _xdata_String_h_

#include <string>
#include "xdata/exception/Exception.h"
#include "xdata/Serializable.h"

namespace xdata {

//template <class T>
class String: public xdata::Serializable
{	
	public:
	
		String(std::string value);
		
		// uninitialized variable
		String();
		
		virtual ~String();
		
		//! Creation from const char *
		String( const char* s );
		
		//! Type cast operator returns the real value
		operator std::string () const;
		
		void setValue(const xdata::Serializable & s)  ;
		
		// Comparison operators inherited from Serializable
		int equals(const xdata::Serializable & s) const;
		
		//! Assignment from const char *
		String& operator=(const char * s);
		
		int operator==(const String & b) const;
				
		int operator!=(const String & b) const;
		
		int operator<(const String & b) const;
		
		int operator<=(const String & b) const;
		
		int operator>(const String & b) const;
		
		int operator>=(const String & b) const;
		
		int operator==(const std::string & s) const;
		
		int operator!=(const std::string & s) const;
		
		int operator<(const std::string & s) const;
		
		int operator>(const std::string & s) const;
		
		
		int operator==(const char* b) const;
		
		int operator!=(const char* b) const;
		
		int operator<(const char* b) const;
		
		int operator>(const char* b) const;
		
		std::string type() const;

		std::string toString () const ;

		std::string toString (bool quoted) const ;

		void  fromString (const std::string& value) ;

		const char* c_str() const;

	
		std::string value_;	
};


} // end namespace xdata

#endif
