// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/

#ifndef _xdata_UnsignedInteger64_h_
#define _xdata_UnsignedInteger64_h_

#include <stdint.h>
#include <string>
#include <errno.h>
#include <limits>
#include "xdata/SimpleType.h"

namespace xdata {

template <class T>
class UnsignedInteger64Type: public xdata::SimpleType<T> 
{
	public:
		//! Initialize with a number
		//
		UnsignedInteger64Type(T value);
		
		//! Uninitialized representing NaN
		//
		UnsignedInteger64Type();
		
		//! CTOR from string containing value
		//
		UnsignedInteger64Type (const std::string & value) 
			;

		virtual ~UnsignedInteger64Type();
		
		//! Prefix ++
		T& operator++();

		//! Prefix --
		T& operator--();

		//! Postfix ++
		T operator++(int);

		//! Postfix --
		T operator--(int);
		
		std::string type() const;

		std::string toString () const ;

		void  fromString (const std::string & value) 
			;			

};

typedef  UnsignedInteger64Type<uint64_t> UnsignedInteger64;
typedef  UnsignedInteger64Type<uint64_t &> UnsignedInteger64Ref;
typedef  uint64_t UnsignedInteger64T;
 
} // end namespace xdata

namespace std {
template<>
struct numeric_limits<xdata::UnsignedInteger64>
{
	static const bool is_specialized = true;

	static xdata::UnsignedInteger64 min() 
	{ 
		return xdata::UnsignedInteger64(std::numeric_limits<xdata::UnsignedInteger64T>::min());
	}

	static xdata::UnsignedInteger64 max() 
	{ 
		return xdata::UnsignedInteger64(std::numeric_limits<xdata::UnsignedInteger64T>::max());

	}

	static const int digits = std::numeric_limits<xdata::UnsignedInteger64T>::digits;
	static const int digits10 = std::numeric_limits<xdata::UnsignedInteger64T>::digits10;
	static const bool is_signed = false;
	static const bool is_integer = true;
	static const bool is_exact = true;
	static const int radix = 2;
	static xdata::UnsignedInteger64 epsilon() 
	{ 
		return xdata::UnsignedInteger64(0); 
	}
	static xdata::UnsignedInteger64 round_error() 
	{ 
		return xdata::UnsignedInteger64(0);
	}

	static const int min_exponent = 0;
	static const int min_exponent10 = 0;
	static const int max_exponent = 0;
	static const int max_exponent10 = 0;

	static const bool has_infinity = true;
	static const bool has_quiet_NaN = true;
	static const bool has_signaling_NaN = false;
	static const float_denorm_style has_denorm = denorm_absent;
	static const bool has_denorm_loss = false;

	static xdata::UnsignedInteger64 infinity() 
	{ 
		xdata::UnsignedInteger64 i(0);
		i.limits_.set(xdata::Infinity);
		return i; 
	}
	static xdata::UnsignedInteger64 quiet_NaN() 
	{ 
		return xdata::UnsignedInteger64(); 
	}

	static xdata::UnsignedInteger64 signaling_NaN() 
	{ 
		return xdata::UnsignedInteger64(0); 
	}

	static xdata::UnsignedInteger64 denorm_min() 
	{ 
		return xdata::UnsignedInteger64(0); 
	}

	static const bool is_iec559 = false;
	static const bool is_bounded = true;
	static const bool is_modulo = true;

	static const bool traps = std::numeric_limits<xdata::UnsignedInteger64T>::traps;
	static const bool tinyness_before = false;
	static const float_round_style round_style = round_toward_zero;
};
} // end of namespace std

#include "xdata/UnsignedInteger64.i"


#endif
