/* i2oAdapterLib.h - i2o Adapter library header file */

/****************************************************************
Copyright 1999 I2O Special Interest Group (I2O SIG).  All rights reserved.

TERMS AND CONDITIONS OF USE

This header file, and any modifications of this header file, are provided
contingent upon your agreement and adherence to the here-listed terms and
conditions.  By accepting and/or using this header file, you agree to
abide by these terms and conditions and that these terms and conditions will
be construed and governed in accordance with the laws of the State of
California, without reference to conflict-of-law provisions.  If you do not
agree to these terms and conditions, please delete this file, and any
copies, permanently, without making any use thereof.

THIS HEADER FILE IS PROVIDED FREE OF CHARGE ON AN AS-IS BASIS WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE.  I2O SIG DOES NOT WARRANT THAT THIS HEADER FILE WILL MEET THE
USER'S REQUIREMENTS OR THAT ITS OPERATION WILL BE UNINTERRUPTED OR
ERROR-FREE.

I2O SIG DISCLAIMS ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF
ANY PROPRIETARY RIGHTS, RELATING TO THE IMPLEMENTATION OF THE I2O
SPECIFICATIONS.  I2O SIG DOES NOT WARRANT OR REPRESENT THAT SUCH
IMPLEMENTATIONS WILL NOT INFRINGE SUCH RIGHTS.

THE USER OF THIS HEADER FILE SHALL HAVE NO RECOURSE TO I2O SIG FOR ANY
ACTUAL OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, LOST DATA
OR LOST PROFITS ARISING OUT OF THE USE OR INABILITY TO USE THIS PROGRAM.

I2O SIG grants the user of this header file a license to copy, distribute,
and modify it, for any purpose, under the following terms.  Any copying,
distribution, or modification of this header file must not delete or alter
the copyright notice of I2O SIG or any of these Terms and Conditions.

Any distribution of this header file must not include a charge for the
header file (unless such charges are strictly for the physical acts of
copying or transferring copies).  However, distribution of a product in
which this header file is embedded may include a charge so long as any
such charge does not include any charge for the header file itself.

Any modification of this header file constitutes a derivative work based
on this header file.  Any distribution of such derivative work: (1) must
include prominent notices that the header file has been changed from the
original, together with the dates of any changes; (2) automatically
includes this same license to the original header file from I2O SIG, without
any restriction thereon from the distributing user; and (3) must include a
grant of license of the modified file under the same terms and conditions as
these Terms and Conditions.

****************************************************************

The I2O SIG Web site can be found at: http://www.i2osig.org

The I2O SIG encourages you to deposit derivative works based on this
header file at the I2O SIG Web site.  Furthermore, to become a Registered
Developer of the I2O SIG, sign up at the Web site or call 415.750.8352
(United States).

****************************************************************/

#ifndef __INCi2oAdapterLibh
#define __INCi2oAdapterLibh

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "i2o/i2o.h"
#include "i2o/i2oBusLib.h"
#include "i2o/shared/i2omsg.h"

/* privates */
typedef struct i2oAdapter*		I2O_ADAPTER_ID;

typedef struct
    {
    union 
        {
        /* PCI Bus */
        I2O_PCI_BUS_INFO        pci;

        /* Local Bus */
        I2O_LOCAL_BUS_INFO      local;

        /* ISA Bus */
        I2O_ISA_BUS_INFO        isa;

        /* EISA Bus */
        I2O_EISA_BUS_INFO       eisa;

        /* MCA Bus */
        I2O_MCA_BUS_INFO        mca;

        /* Other. */
        I2O_OTHER_BUS_INFO      other;
        } u;

    } I2O_PHYSICAL_LOCATION, *PI2O_PHYSICAL_LOCATION;

/* function declarations */

extern I2O_BUS_ID             i2oAdapterBusGet (I2O_ADAPTER_ID  adapterId,
                                                I2O_STATUS*     pStatus);

extern void i2oAdapterPhysLocGet (I2O_ADAPTER_ID                adapterId,
                                  I2O_PHYSICAL_LOCATION*        pPhys,
                                  I2O_STATUS*                   pStatus);

extern void i2oAdapterIntLock (I2O_ADAPTER_ID  adapterId,
                               I2O_STATUS*     pStatus);

extern void i2oAdapterIntUnlock (I2O_ADAPTER_ID  adapterId,
                                 I2O_STATUS*     pStatus);

extern U8        i2oAdapterConfigRead8(I2O_ADAPTER_ID        adapterId,
                                       I2O_ADDR32         offset,
                                       I2O_STATUS*        pStatus);
extern U16       i2oAdapterConfigRead16(I2O_ADAPTER_ID       adapterId,
                                        I2O_ADDR32        offset,
                                        I2O_STATUS*       pStatus);
extern U32       i2oAdapterConfigRead32(I2O_ADAPTER_ID       adapterId,
                                        I2O_ADDR32        offset,
                                        I2O_STATUS*       pStatus);
extern U64       i2oAdapterConfigRead64(I2O_ADAPTER_ID       adapterId,
                                        I2O_ADDR32        offset,
                                        I2O_STATUS*       pStatus);
extern void      i2oAdapterConfigWrite8(I2O_ADAPTER_ID    adapterId,
                                        I2O_ADDR32        offset,
                                        U8                value,
                                        I2O_STATUS*       pStatus);
extern void      i2oAdapterConfigWrite16(I2O_ADAPTER_ID   adapterId,
                                         I2O_ADDR32       offset,
                                         U16              value,
                                         I2O_STATUS*      pStatus);
extern void      i2oAdapterConfigWrite32(I2O_ADAPTER_ID   adapterId,
                                         I2O_ADDR32       offset,
                                         U32              value,
                                         I2O_STATUS*      pStatus);
extern void      i2oAdapterConfigWrite64(I2O_ADAPTER_ID   adapterId,
                                         I2O_ADDR32       offset,
                                         U64              value,
                                         I2O_STATUS*      pStatus);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INCi2oAdapterLibh */
