// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/

#include "toolbox/hexdump.h"
#include "toolbox/string.h"

#include <stdio.h>
#include <ctype.h>
#include <string.h>

void toolbox::hexdump(std::ostream& out, std::string fmt, void* data, size_t size)
{
  /* dumps size bytes of *data to stdout. Looks like:
  
  	32 bit platform:
        [0000] 75 6E 6B 6E 6F 77 6E 20 30 FF 00 00 00 00 39 00 unknown 0.....9.
	
	64bit platform:
	[00000001] 75 6E 6B 6E 6F 77 6E 20 30 FF 00 00 00 00 39 00 unknown 0.....9.0.....9.
  */

    unsigned char *p = (unsigned char*) data;
    unsigned char c;
    size_t n;
    char bytestr[4] = {0};
    char addrstr[10] = {0};
    char hexstr[ 16*3 + 5] = {0};
    char charstr[16*1 + 5] = {0};
    
    for(n=1;n<=size;n++) 
    {
	if (n%16 == 1) 
	{
		// store address for this line
#if ( __WORDSIZE == 32 )
		snprintf(addrstr, sizeof(addrstr), "%.4x",
			((unsigned int)p-(unsigned int)data) );
#elif ( __WORDSIZE == 64 )
		snprintf(addrstr, sizeof(addrstr), "%.8lx",
			((unsigned long)p-(unsigned long)data) );
#else
		return; // other wordsize not supported
#endif
        }
            
        c = *p;
        if (isalnum(c) == 0) 
	{
            c = '.';
        }

        /* store hex str (for left side) */
        snprintf(bytestr, sizeof(bytestr), "%02X ", *p);
        strncat(hexstr, bytestr, sizeof(hexstr)-strlen(hexstr)-1);

        /* store char str (for right side) */
        snprintf(bytestr, sizeof(bytestr), "%c", c);
        strncat(charstr, bytestr, sizeof(charstr)-strlen(charstr)-1);

        if(n%16 == 0) { 
            /* line completed */
#if ( __WORDSIZE == 32 )	  
            out << toolbox::toString("[%4.4s]   %-50.50s  %s", addrstr, hexstr, charstr) << std::endl;
#elif ( __WORDSIZE == 64 )
	    out << toolbox::toString("[%8.8s]   %-50.50s  %s", addrstr, hexstr, charstr) << std::endl;
#else
	    out << "Wordsize " << __WORDSIZE << " not supported" << std::endl;
#endif		    	    
            
            hexstr[0] = 0;
            charstr[0] = 0;
        } else if(n%8 == 0) {
            /* half line: add whitespaces */
            strncat(hexstr, "  ", sizeof(hexstr)-strlen(hexstr)-1);
            strncat(charstr, " ", sizeof(charstr)-strlen(charstr)-1);
        }
        p++; /* next byte */
    }

    if (strlen(hexstr) > 0) {
        /* print rest of buffer if not empty */
         out << toolbox::toString("[%4.4s]   %-50.50s  %s", addrstr, hexstr, charstr) << std::endl;
    }
}

void toolbox::hexdump(void *data, size_t size)
{
    /* dumps size bytes of *data to stdout. Looks like:
       [0000] 75 6E 6B 6E 6F 77 6E 20 30 FF 00 00 00 00 39 00 unknown 0.....9.
     */

    unsigned char *p = (unsigned char*) data;
    unsigned char c;
    size_t n;
    char bytestr[4] = {0};
    char addrstr[10] = {0};
    char hexstr[ 16*3 + 5] = {0};
    char charstr[16*1 + 5] = {0};
    for(n=1;n<=size;n++) 
    {
        if (n%16 == 1) 
	{
		// store address for this line
#if ( __WORDSIZE == 32 )
		snprintf(addrstr, sizeof(addrstr), "%.4x",
			((unsigned int)p-(unsigned int)data) );
#elif ( __WORDSIZE == 64 )
		snprintf(addrstr, sizeof(addrstr), "%.8lx",
			((unsigned long)p-(unsigned long)data) );
#else
		return; // other wordsize not supported
#endif
        }
            
        c = *p;
        if (isalnum(c) == 0) {
            c = '.';
        }

        /* store hex str (for left side) */
        snprintf(bytestr, sizeof(bytestr), "%02X ", *p);
        strncat(hexstr, bytestr, sizeof(hexstr)-strlen(hexstr)-1);

        /* store char str (for right side) */
        snprintf(bytestr, sizeof(bytestr), "%c", c);
        strncat(charstr, bytestr, sizeof(charstr)-strlen(charstr)-1);

        if(n%16 == 0) { 
            /* line completed */
#if ( __WORDSIZE == 32 )	  
            printf("[%4.4s]   %-50.50s  %s\n", addrstr, hexstr, charstr);
#elif ( __WORDSIZE == 64 )
	    printf("[%8.8s]   %-50.50s  %s\n", addrstr, hexstr, charstr);
#else
		printf("Wordsize %ul no supported\n", __WORDSIZE);
#endif	    
            hexstr[0] = 0;
            charstr[0] = 0;
        } else if(n%8 == 0) {
            /* half line: add whitespaces */
            strncat(hexstr, "  ", sizeof(hexstr)-strlen(hexstr)-1);
            strncat(charstr, " ", sizeof(charstr)-strlen(charstr)-1);
        }
        p++; /* next byte */
    }

    if (strlen(hexstr) > 0) {
        /* print rest of buffer if not empty */
        printf("[%4.4s]   %-50.50s  %s\n", addrstr, hexstr, charstr);
    }
}


