// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/

#ifndef _xdaq_ContextTable_h_
#define _xdaq_ContextTable_h_

#include <string>
#include <map>
#include <vector>

#include "xdaq/ContextDescriptor.h"
#include "xdaq/exception/HostNotFound.h"
#include "xdaq/exception/DuplicateHost.h"
#include "xdaq/exception/CreationFailed.h"

namespace xdaq 
{
	class ContextTable
	{
		public:

		// DTOR
		~ContextTable();

		//! Create a new context
		/*! A context is uniquely identified by its URL, e.g. hostname and port 
		 *  A sessionId different from empty string "" may be provided. It can be used
		 *   to associate a running context (= a session) with other running contexts
		 */
		xdaq::ContextDescriptor* createContextDescriptor (const std::string& url) 
			;

		//! Remove a context descriptor by URL
 		void removeContext(const std::string & url) ;

		//! Retrieve a ContextDescriptor by indicating the URL
 		const xdaq::ContextDescriptor*  getContextDescriptor(const std::string & url) const ;

		//! Retrieve all context descriptors
 		std::vector<xdaq::ContextDescriptor*>  getContextDescriptors() const;


		bool hasContextDescriptor(const std::string & url) const;

		private:

		std::map<std::string, xdaq::ContextDescriptor*, std::less<std::string> > contexts_;

	};
}

#endif
