// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/
#include <stdlib.h>
#include <cmath>
#include <float.h>

template<class T> xdata::DoubleType<T>::DoubleType(T value):xdata::SimpleType<T>(value)
{
	if ( std::isinf(value) != 0 )
	{
		this->limits_.set(xdata::Infinity);
	}
	else if ( std::isnan(value) != 0 )
	{
		this->limits_.set(xdata::NaN);
	}
}

// uninitialized variable
template<class T> xdata::DoubleType<T>::DoubleType()
{		
}

template<class T> xdata::DoubleType<T>::DoubleType(const std::string & value) 
{
	this->fromString(value);
}


template<class T> xdata::DoubleType<T>::~DoubleType()
{		
}


template<class T> std::string xdata::DoubleType<T>::type() const
{
		return "double";
}

template<class T> std::string xdata::DoubleType<T>::toString () const 
{
	if (this->isInf())
	{
		return "infinite";
	}
	else if (this->isNaN())
	{
		return "NaN";
	}
	else
	{
		char tmp[255];
        sprintf (tmp, "%.*e", DBL_DIG - 1, this->value_);
        return tmp;
	}	
}

template<class T> void  xdata::DoubleType<T>::fromString (const std::string & value) 
{
	this->limits_.reset(); // reset any bits
	if ((value == "") || (value == "NaN"))
	{
		// empty string is not a number
		this->limits_.set(xdata::NaN);
                this->value_ = std::numeric_limits<xdata::DoubleT>::quiet_NaN();
		return;
	}
	else if (value == "infinite")
	{
		this->limits_.set(xdata::Infinity);
                this->value_ = std::numeric_limits<xdata::DoubleT>::infinity();
		return;
	}
	//sscanf (value.c_str(), "%lf", &value_);
	// The following code will convert an integer
	// from any represenation, e.g. hex, oct, bin or decimal
	//
	char* firstWrong = 0;

	errno = 0;
	this->value_ = strtod ( value.c_str(), &firstWrong );

	if (	(errno == ERANGE) || (*firstWrong != '\0') )
	{
		this->limits_.set(xdata::NaN);
		std::string msg = "Invalid double format. Value ";
		msg += value;
		msg += ", error at ";
		msg += firstWrong;
		XCEPT_RAISE (xdata::exception::Exception, msg);
	}
}	

