// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2009, CERN.			                 *
 * All rights reserved.                                                  *
 * Authors: J. Gutleber and L. Orsini					 *
 *                                                                       *
 * For the licensing terms see LICENSE.		                         *
 * For the list of contributors see CREDITS.   			         *
 *************************************************************************/
#include <stdlib.h>
/*template<class T>  xdata::SimpleType<T>::SimpleType (const std::string & value) 
{

}
*/

template<class T> xdata::SimpleType<T>::SimpleType ( T value) :  value_(value)
{
	limits_.reset();
}

template<class T> xdata::SimpleType<T>::SimpleType ()
{
	limits_.set(xdata::NaN);
}


template<class T> xdata::SimpleType<T>::~SimpleType() 
{
}

//! Type cast operator returns the real value
template<class T> xdata::SimpleType<T>::operator T () const
{ 
	return value_; 
}

/*
template<class T> T & xdata::SimpleType<T>::operator *() 
{ 
	return *this; 
}
*/

// virtual assignement operator
template<class T>  void xdata::SimpleType<T>::setValue(const xdata::Serializable & s)  
{	
    // this trigger recursion *this = dynamic_cast<const SimpleType<T> &>(s);	
    try
    {
	const SimpleType<T> & v =  dynamic_cast<const SimpleType<T> &>(s);
	value_ = v.value_;
	limits_ = v.limits_;
    }
    catch(std::bad_cast const& exp)
    {
	XCEPT_RAISE(xdata::exception::Exception, exp.what());
    }
}

//! C++ types compatible Assignment operator
template<class T> xdata::SimpleType<T>& xdata::SimpleType<T>::operator=(const T & value)
{
	//std::cout << "template<class T> xdata::SimpleType<T>& xdata::SimpleType<T>::operator=(const T & value)" << std::endl;
	value_ = value;
	limits_.reset();
	return *this;
}

// ------------------------------------------------------------------

template<class T> int xdata::SimpleType<T>::operator==(const SimpleType<T> & b) const
{
	if ( this->isFinite() && b.isFinite() )
		return (value_ == b.value_);
	else
		return false;	
}

template<class T> int xdata::SimpleType<T>::operator!=(const SimpleType<T> & b) const
{
	if ( this->isFinite() && b.isFinite() )
		return (value_ != b.value_);
	else
		return false;	
}

template<class T> int xdata::SimpleType<T>::operator<(const SimpleType<T> & b) const
{
	if ( this->isFinite() && b.isFinite() )
		return (value_ < b.value_);
	else
		return false;
}

template<class T> int xdata::SimpleType<T>::operator<=(const SimpleType<T> & b) const
{
	if ( this->isFinite() && b.isFinite() )
		return (value_ <= b.value_);
	else
		return false;		
}

template<class T> int xdata::SimpleType<T>::operator>(const SimpleType<T> & b) const
{
	if ( this->isFinite() && b.isFinite() )
		return (value_ > b.value_);
	else
		return false;		
}

template<class T> int xdata::SimpleType<T>::operator>=(const SimpleType<T> & b) const
{
	if ( this->isFinite() && b.isFinite() )
		return (value_ >= b.value_);
	else
		return false;			
}

// ------------------------------------------------------------------

template<class T> int xdata::SimpleType<T>::operator==(const T & b) const
{
	if ( this->isFinite() )
		return (value_ == b);
	else
		return false;
}

template<class T> int xdata::SimpleType<T>::operator!=(const T & b) const
{
	if ( this->isFinite() )
		return (value_ != b);
	else
		return false;
}

template<class T> int xdata::SimpleType<T>::operator!() const
{
	if ( this->isFinite() )
		return (!value_);
	else
		return false;	
}

template<class T> int xdata::SimpleType<T>::operator<(const T & b) const
{
	if ( this->isFinite() )
		return (value_ < b);
	else
		return false;	
}

template<class T> int xdata::SimpleType<T>::operator<=(const T & b) const
{
	if ( this->isFinite() )
		return (value_ <= b);
	else
		return false;	
}

template<class T> int xdata::SimpleType<T>::operator>(const T & b) const
{
	if ( this->isFinite() )
		return (value_ > b);
	else
		return false;	
}

template<class T> int xdata::SimpleType<T>::operator>=(const T & b) const
{
	if ( this->isFinite() )
		return (value_ >= b);
	else
		return false;	
}

// ------------------------------------------------------------------

template<class T> int xdata::SimpleType<T>::equals(const xdata::Serializable & s) const
{
	return (*this == dynamic_cast<const xdata::SimpleType<T>&>(s));
}

// ------------------------------------------------------------------
/*template<class T> T& xdata::SimpleType<T>::operator++()
{
	return (++value_);
}

template<class T> T& xdata::SimpleType<T>::operator--()
{
	return (--value_);
}

template<class T> T xdata::SimpleType<T>::operator++(int)
{
	value_++;
	return value_;
}

template<class T> T xdata::SimpleType<T>::operator--(int)
{
	value_--;
	return value_;
}
*/
